/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.api;

import java.rmi.RemoteException;
import java.util.ArrayList;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.bpe.api.ProcessException;
import com.ibm.bpe.api.QueryResultSet;
import com.ibm.bpe.cm.ejb.ContentManagerIntegration;
import com.ibm.bpe.cm.ejb.ContentManagerIntegrationHome;
import com.ibm.bpe.cm.util.IntegrationException;
import com.ibm.bpe.framework.ProcessBase;
import com.ibm.mm.sdk.common.DKException;
import com.ibm.mm.sdk.common.DKPidICM;

/**
 * This class provides access to the Content Manager / Process Choreographer
 * integration function. It implements the "business delegate" pattern to
 * hide any complexities involved with lookup of an EJB which eventually contains
 * the real implementation.<br> 
 * Use these static functions inside a Process Choreographer Java snippet
 * or inside a Java transition condition.<br>
 * The integration code makes some assumptions about process attributes,
 * the most important one is the Content Manager itemPID of an associated
 * document or document folder. Use <code>initializeCustomProperties</code>
 * at the very beginning of your business process to set these properties properly.<br>
 */
public class ContentManager {

	/**
	 * Evaluate a Content Manager XPath expression.
	 * Use this function in a Process Choreographer transition condition to
	 * determine a flow route based on document contents.<br>
	 * For a given item type "T1" with an attribut "a1", a valid XPath expression would
	 * be "/T1[@a1='xy']".<br> 
	 * If you want to refer to the document or folder associated with the
	 * business process, you may want to use the placeholder
	 * defined by <code>SUBSTITUTABLE_PARAMETER_XXX</code> to refer to itemId, componentId
	 * and version number of the item.
	 * 
	 * @param processBean A reference to the bean holding the Java implementation in a business process
	 *   In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param xPath A valid Content Manager XPath expression for a condition with optional substitutable parameters
	 * @return true, if the condition is true
	 */
	static public boolean evaluateCondition(
		ProcessBase processBean,
		String xPath) {
		xPath = substituteParameters(processBean, xPath);
		boolean conditionIsTrue = false;
		ContentManagerIntegration contentManagerIntegrationEjb =
			getContentManagerIntegrationEjb();
		if (contentManagerIntegrationEjb != null) {
			try {
				conditionIsTrue =
					contentManagerIntegrationEjb.evaluateCondition(xPath);
			} catch (RemoteException exc) {
				throw new IntegrationException(exc);
			}
		}
		return conditionIsTrue;
	}

	/**
	 * In order to refer to the item associated with the business process, you
	 * may want to use placeholders defined with the <code>SUBSTITUTABLE_PARAMETER_XXX</code>
	 * strings.<br>
	 * This function replaces the placeholders by its real value associated with the
	 * business process.<br>
	 * 
	 * @param processBean A reference to the bean holding the Java implementation in a business process
	 *   In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param xPath A Content Manager XPath expression with optional substitutable parameters
	 * @return The xPath String with substitutable parameters replaced by real values
	 */
	static String substituteParameters(ProcessBase processBean, String xPath) {
		String itemId =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_ITEMID);
		String componentId =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_COMPONENTID);
		String versionId =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_VERSIONID);
		String itemType =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_ITEMTYPE);
		xPath =
			xPath.replaceAll(
				SUBSTITUTABLE_PARAMETER_ITEMID,
				"\"" + itemId + "\"");
		xPath =
			xPath.replaceAll(
				SUBSTITUTABLE_PARAMETER_COMPONENTID,
				"\"" + componentId + "\"");
		xPath = xPath.replaceAll(SUBSTITUTABLE_PARAMETER_VERSIONID, versionId);
		xPath =
			xPath.replaceAll(
				SUBSTITUTABLE_PARAMETER_ITEMTYPE,
				"\"" + itemType + "\"");
		return xPath;
	}

	/**
	 * Get an attribute value of the item associated with the business process. 
	 * @param processBean A reference to the bean holding the Java implementation in a business process
	 *   In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param attributeSpecification Attribute to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
	 * format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
	 * Note that &lt;attribute-name&gt; can include child component names,
	 * e.g. Book/Chapter/Section/Author.
	 * @return An Object for the attribute value or null if the attribute cannot be resolved.
	 * @see #getContentAttributes(ProcessBase, String[])
	 * @see #getFolderContentAttribute(ProcessBase, String)		
	 * @see #getFolderContentAttributes(ProcessBase, String []) 
	 */
	public static Object getContentAttribute(
		ProcessBase processBean,
		String attributeSpecification) {
		String itemPidString =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_ITEMPID);
		ContentManagerIntegration contentManagerIntegrationEjb =
			getContentManagerIntegrationEjb();
		if (contentManagerIntegrationEjb != null) {
			try {
				return contentManagerIntegrationEjb.getContentAttribute(
					itemPidString,
					attributeSpecification);
			} catch (RemoteException exc) {
				throw new IntegrationException(exc);
			}
		}
		return null;
	}

	/**
	 * Get multiple attributes of the item associated with the business process.
	 * @param processBean A reference to the bean holding the Java implementation in a business process
	 *   In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param attributeSpecifications Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; format, e.g.
	 * <code><pre>
	 * new String[]
	 * {
	 *     "XYZ_ClaimForm/XYZ_DriversLic",
	 *     "XYZ_ClaimForm/XYZ_PolicyNum",
	 *     "XYZ_AdjReport/XYZ_LicPlate"
	 * }
	 * </pre></code>
	 * Note that &lt;attribute-name&gt; can include child component names,
	 * e.g. Book/Chapter/Section/Author.
	 * @return An ArrayList of ArrayLists (e.g. a table) containing
	 * one row for each returned attribute and columns for item type
	 * name, item pid string, attribute name and attribute value.
	 * @see #getContentAttribute(ProcessBase, String)
	 * @see #getFolderContentAttribute(ProcessBase, String) 
	 * @see #getFolderContentAttributes(ProcessBase, String[]) 
	 */
	public static ArrayList getContentAttributes(
		ProcessBase processBean,
		String[] attributeSpecifications) {
		String itemPidString =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_ITEMPID);
		ContentManagerIntegration contentManagerIntegrationEjb =
			getContentManagerIntegrationEjb();
		if (contentManagerIntegrationEjb != null) {
			try {
				return contentManagerIntegrationEjb.getContentAttributes(
					itemPidString,
					attributeSpecifications);
			} catch (RemoteException exc) {
				throw new IntegrationException(exc);
			}
		}
		return null;
	}

	/**
	 * Get an attribute value of item in the folder which is associated with the business process<br>
	 * Typically, the business process is associated with a folder rather than a single item.
	 * The folder then contains one or more items - use this functions to retrieve a single attributes
	 * from such an item inside a folder (e.g. the license plate attribute in the adjuster's report item type).<br>
	 * If the business process is not associated with a folder or if you want to get attributes of
	 * the folder itself, use <code>getContentAttribute</code> instead.
	 * Note that attributes of the latest version are retrieved.
     * @param processBean A reference to the bean holding the Java implementation in a business process
	 * In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param attributeSpecification Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
	 * format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
	 * Note that &lt;attribute-name&gt; can include child component names,
	 * e.g. Book/Chapter/Section/Author.
	 * @return An Object for the attribute value or null if the attribute cannot be resolved.		
	 * @see #getFolderContentAttributes(ProcessBase, String[])
	 * @see #getContentAttribute(ProcessBase, String)
	 * @see #getContentAttributes(ProcessBase, String []) 
	 */
	public static Object getFolderContentAttribute(
		ProcessBase processBean,
		String attributeSpecification) {
		String folderPidString =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_ITEMPID);
		ContentManagerIntegration contentManagerIntegrationEjb =
			getContentManagerIntegrationEjb();
		if (contentManagerIntegrationEjb != null) {
			try {
				return contentManagerIntegrationEjb.getFolderContentAttribute(
					folderPidString,
					attributeSpecification);
			} catch (RemoteException exc) {
				throw new IntegrationException(exc);
			}
		}
		return null;
	}

	/**
	 * Get multiple attribute values of items in the folder which is associated with the business process<br>
	 * Typically, the business process is associated with a folder rather than a single item.
	 * The folder then contains one or more items - use this functions to retrieve attributes
	 * from such items inside a folder (e.g. the license plate attribute in the adjuster's report item type).<br>
	 * If the business process is not associated with a folder or if you want to get attributes of
	 * the folder itself, use <code>getContentAttributes</code> instead.
	 * Note that attributes of the latest version are retrieved.
     * @param processBean A reference to the bean holding the Java implementation in a business process
	 * In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param attributeSpecifications Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
	 * format, e.g.
	 * <code><pre>
	 * new String[]
	 * {
	 *     "XYZ_ClaimForm/XYZ_DriversLic",
	 *     "XYZ_ClaimForm/XYZ_PolicyNum",
	 *     "XYZ_AdjReport/XYZ_LicPlate"
	 * }
	 * </pre></code>
	 * Note that &lt;attribute-name&gt; can include child component names,
	 * e.g. Book/Chapter/Section/Author.
	 * @return An ArrayList of ArrayLists (e.g. a table) containing
	 * one row for each returned attribute and columns for item type
	 * name, item pid string, attribute name and attribute value.   
	 * @see #getFolderContentAttribute(ProcessBase, String)
	 * @see #getContentAttribute(ProcessBase, String)
	 * @see #getContentAttributes(ProcessBase, String[])
	 */
	static public ArrayList getFolderContentAttributes(
		ProcessBase processBean,
		String[] attributeSpecifications) {
		String folderPidString =
			processBean.getProcessCustomProperty(
				ContentManager.PROCESS_PROPERTY_ITEMPID);
		ContentManagerIntegration contentManagerIntegrationEjb =
			getContentManagerIntegrationEjb();
		if (contentManagerIntegrationEjb != null) {
			try {
				return contentManagerIntegrationEjb.getFolderContentAttributes(
					folderPidString,
					attributeSpecifications);
			} catch (RemoteException exc) {
				throw new IntegrationException(exc);
			}
		}
		return null;
	}

	/**
	 * Run a combined query for Process Choreographer and Content Manager data.
	 * For more details on the QueryResultSet and how to specify the clauses, also refer
	 * to the Process Choreographer documentation.<br>
	 * <b>Note:</b> If you plan to use this function from an EJB or Servlet context, consider 
	 * calling <code>combinedQuery</code> on the <code>ContentManagerIntegrationEjb</code>
	 * instead using local EJB calls. This may give you better performance than through this
	 * business delegate class.<br>
	 *   
	 * @param selectClauseWorkflow The requested Process Choreographer column names. Must not be null.
	 * @param flowItemTypeNameContent The item type of the item which is routed through the flow
	 * This can be a document item type or a folder item type. Must not be null.
	 * @param flowItemTypeAttributesContent A comma separated list of attributes of the flow item type
	 * that should be returned in the result set. 
	 * @param additionalAttributesContent If the flowItemTypeContent is a folder, this specifies an 
	 * optional list of attributes of the contained documents. The list is comma separated and
	 * the attributes are specified in the form <code>&lt;itemtype&gt;/&lt;attribute&gt;</code>.   
	 * @param whereClauseWorkflow The SQL where condition for Process Choreographer data, may be null.
	 * @param xPathContent  An valid XPath expression to filter the Content Manager data. The result must
	 * be the flow item type. To express conditions on contained item attributes make sure
	 * to use an inboundlink to get to the folder item. 	  	
	 * @param orderByClauseWorkflow The order clause for Process Choreographer data, may be null.
	 * @param timezoneWorkflow A timezone specification for timestamp given or returned, may be null. 
	 * @return The result of the combined query.
	 */
	static public QueryResultSet combinedQuery(
		String selectClauseWorkflow,
		String flowItemTypeNameContent,
		String flowItemTypeAttributesContent,
		String additionalAttributesContent,
		String whereClauseWorkflow,
		String xPathContent,
		String orderByClauseWorkflow,
		java.util.TimeZone timezoneWorkflow) {
		ContentManagerIntegration contentManagerIntegrationEjb =
			getContentManagerIntegrationEjb();
		if (contentManagerIntegrationEjb != null) {
			try {
				return contentManagerIntegrationEjb.combinedQuery(
					selectClauseWorkflow,
					flowItemTypeNameContent,
					flowItemTypeAttributesContent,
					additionalAttributesContent,
					whereClauseWorkflow,
					xPathContent,
					orderByClauseWorkflow,
					timezoneWorkflow);
			} catch (RemoteException exc) {
				throw new IntegrationException(exc);
			}
		}

		return null;
	}

	/**
	  * Use this function in a Java snippet at the very beginning of your
	  * business process to associate it with a Content Manager item.
	  * Information about an item could either be an <code>itemPID</code> string
	  * or values for <code>itemId, componentId and versionId</code>.<br>
	  * You typically get this reference information to a Content Manager
	  * item from an input variable of the business process.<br>
	  * This function sets certain process attribute values with item reference data
	  * required for other functions in this integration code.   
	  * @param processBean A reference to the bean holding the Java implementation in a business process
	  *   In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
		* @param itemId The itemId value of a Content Manager document or folder.
		* @param componentId The componentId value of a Content Manager document or folder.
		* @param versionId The versionId value of a Content Manager document or folder.
		* @see #initializeCustomProperties(ProcessBase, String)  
		*/
	public static void initializeCustomProperties(
		ProcessBase processBean,
		String itemId,
		String componentId,
		int versionId) {
		try {
			String pidString =
				getContentManagerIntegrationEjb().getPid(
					itemId,
					componentId,
					versionId);
			DKPidICM pid = new DKPidICM(pidString);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_ITEMPID,
				pidString);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_ITEMID,
				itemId);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_COMPONENTID,
				componentId);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_VERSIONID,
				Integer.toString(versionId));
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_ITEMTYPE,
				pid.getObjectType());
		} catch (Exception exc) {
			throw new IntegrationException(exc);
		}

	}

	/**
	 * Use this function in a Java snippet at the very beginning of your
	 * business process to associate it with a Content Manager item.
	 * Information about an item could either be an <code>itemPID</code> string
	 * or values for <code>itemId, componentId and versionId</code>.<br>
	 * You typically get this reference information to a Content Manager
	 * item from an input variable of the business process.<br>
	 * This function sets certain process attribute values with item reference data
	 * required for other functions in this integration code.   
	 * @param processBean A reference to the bean holding the Java implementation in a business process
	 *   In a Java snippet or in a Java transition condition, use <code>this</code> to get this reference. 
	 * @param itemPid The itemPID string value of a Content Manager document or folder.
	 * @see #initializeCustomProperties( ProcessBase, String, String , int )
	 */
	public static void initializeCustomProperties(
		ProcessBase processBean,
		String itemPid) {
		try {
			DKPidICM pid = new DKPidICM(itemPid);
			String itemId = pid.getItemId();
			String componentId = pid.getComponentId();
			String versionId = pid.getVersionNumber();
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_ITEMPID,
				itemPid);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_ITEMID,
				itemId);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_COMPONENTID,
				componentId);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_VERSIONID,
				versionId);
			processBean.setProcessCustomProperty(
				PROCESS_PROPERTY_ITEMTYPE,
				pid.getObjectType());
		} catch (ProcessException exc) {
			throw new IntegrationException(exc);
		} catch (DKException exc) {
			throw new IntegrationException(exc);
		}
	}

	/**
	* Internal function to cache the home interface of the Integration Toolkit
	* EJB and create a new instance of the EJB for one of the above service calls.
	* Note: since this function is called from the context of a Process Choreographer
	* process, there are no resource ref's present which enables us to use the local EJB calls.
	* Thus, make a global remote EJB lookup here.
	* @return A remote interface for the Integration Toolkit EJB
	*/
	static private ContentManagerIntegration getContentManagerIntegrationEjb() {
		try {
			if (_contentManagerIntegrationHome == null) {
				synchronized (_semaphore) {
					// double-check pattern
					if (_contentManagerIntegrationHome == null) {
						InitialContext initialContext = new InitialContext();
						_contentManagerIntegrationHome =
							(
								ContentManagerIntegrationHome) initialContext
									.lookup(
								"ejb/com/ibm/bpe/cm/ejb/ContentManagerIntegrationHome");
					}
				}
			}
			if (_contentManagerIntegrationHome != null) {
				return _contentManagerIntegrationHome.create();
			}
		} catch (NamingException exc) {
			throw new IntegrationException(exc);
		} catch (CreateException exc) {
			throw new IntegrationException(exc);
		} catch (RemoteException exc) {
			throw new IntegrationException(exc);
		}
		return null;
	}

	/**
	 * Semaphore object which is used to synchronized creation of the home interface.
	 */
	static private Object _semaphore = new Object();
	static private ContentManagerIntegrationHome _contentManagerIntegrationHome =
		null;

	/**
	 * Name of the custom process property to store the associated itemPid string.
	 */
	public static final String PROCESS_PROPERTY_ITEMPID = "itemPid";

	/**
	 * Name of the custom process property to store the associated itemId string.
	 */
	public static final String PROCESS_PROPERTY_ITEMID = "itemId";

	/**
	 * Name of the custom process property to store the associated componentId string.
	 */
	public static final String PROCESS_PROPERTY_COMPONENTID = "componentId";

	/**
	 * Name of the custom process property to store the associated versionId string.
	 */
	public static final String PROCESS_PROPERTY_VERSIONID = "versionId";

	/**
	 * Name of the custom process property to store the associated itemType string.
	 */
	public static final String PROCESS_PROPERTY_ITEMTYPE = "itemType";

	/**
	 * Name of XPath query expression Item ID substitutable parameters
	 */
	public static final String SUBSTITUTABLE_PARAMETER_ITEMID =
		"%" + PROCESS_PROPERTY_ITEMID.toUpperCase() + "%";

	/**
	 * Name of XPath query expression Component ID substitutable parameters
	 */
	public static final String SUBSTITUTABLE_PARAMETER_COMPONENTID =
		"%" + PROCESS_PROPERTY_COMPONENTID.toUpperCase() + "%";

	/**
	 * Name of XPath query expression Version ID substitutable parameters
	 */
	public static final String SUBSTITUTABLE_PARAMETER_VERSIONID =
		"%" + PROCESS_PROPERTY_VERSIONID.toUpperCase() + "%";

	/**
	 * Name of XPath query expression Item Type ID substitutable parameters
	 */
	public static final String SUBSTITUTABLE_PARAMETER_ITEMTYPE =
		"%" + PROCESS_PROPERTY_ITEMTYPE.toUpperCase() + "%";

	/**
	 * Return value for the Collection Point Service:
	 * The condition is satisfied and processing of the request ended regularly. 
	 */
	public static int COLLECTIONPOINT_OK = 0;

	/**
	  * Return value for the Collection Point Service:
	  * An error occurred during collection point processing. Check the log for details. 
	  */
	public static int COLLECTIONPOINT_ERROR = -1;

	/**
	 * Return value for the Collection Point Service:
	 * The specified timeout is reached. 
	 */
	public static int COLLECTIONPOINT_TIMEOUT = 1;

	/**
	 * Return value for the Collection Point Service:
	 * The associated item has been deleted. 
	 */
	public static int COLLECTIONPOINT_ITEM_DELETED = 2;
}
