/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import com.ibm.bpe.api.OID;
import com.ibm.bpe.api.QueryColumnInfo;
import com.ibm.bpe.api.QueryResultSet;
import com.ibm.bpe.api.UTCDate;

/**
 * Combine a WPC result set with a CM query result set.
 * The constructor is used to gather sizes and meta information,
 * the <code>add</code> function is used to add new rows to the result set.
 * An object of this class is returned as a result of a combined query.<br>
 * Note: this version assumes that the CM contribution are only Strings.
 * Numerical values are translated into Strings.
 */
public class CombinedQueryResultSet implements QueryResultSet
{
  /**
   * Create a new combined result set for queries.
   * @param resultSetWpc A result set from a WPC query
   * @param columnNamesCm A list of Strings with the columns names (headings) for the CM part
   * @param timezone (optional) The timezone if the query expression had TS() attributes.
   */
  public CombinedQueryResultSet(
    QueryResultSetAdapter resultSetWpc,
    List columnNamesCm,
    TimeZone timezone)
  {
    // The number of columns from the WPC, CM part and the total number of columns		
    _numberColumnsWpc = resultSetWpc.numberColumns();
    _numberColumnsCm = columnNamesCm.size();
    _numberColumns = _numberColumnsWpc + _numberColumnsCm;

    _timezone = timezone;

    // One row of the result set will be one entry in this ArrayList
    _result = new ArrayList();

    // Assign the variables for the display names (headers) and 
    // information about the columns (type, name, etc.)
    _columnDisplayNames = new String[_numberColumns];
    _selectedColumns = new ArrayList();
    // Copy information for the WPC piece
    for (int i = 0; i < _numberColumnsWpc; i++)
    {
      String columnDisplayName = resultSetWpc.getColumnDisplayName(i + 1);
      short columnType = resultSetWpc.getColumnType(i + 1);
      _columnDisplayNames[i] = columnDisplayName;
      QueryColumnInfo columnInfo = new QueryColumnInfo(columnDisplayName, columnType, true);
      _selectedColumns.add(columnInfo);
    }
    // Copy information for the CM piece
    for (int i = _numberColumnsWpc; i < _numberColumns; i++)
    {
      String columnDisplayName = (String) columnNamesCm.get(i - _numberColumnsWpc);
      short columnType = QueryColumnInfo.TYPE_STRING;
      _columnDisplayNames[i] = columnDisplayName;
      QueryColumnInfo columnInfo = new QueryColumnInfo(columnDisplayName, columnType, true);
      _selectedColumns.add(columnInfo);
    }
  }

  /**
   * Add a WPC resultSet row and a list of CM columns to the combined result set.
   * @param columnsWpc A row from a WPC result set (more precisely - the entire result set with the current 
   * cursor pointing to a particular row)
   * @param columnsCm A list of Strings, representing the associated CM column values.
   */
  public void add(QueryResultSetAdapter columnsWpc, List columnsCm)
  {    
    // Allocate a new result entry with the size of selected columns
    Object[] resultEntry = new Object[_numberColumns];

    //
    // Assign the WPC columns to resultEntry[] - note that IDs are handled differently
    // since QueryResultSetImpl in the WPC code has to care about Informix which cannot store
    // binaries. Thus, get the OID object and store it in the entry array.
    //
    for (int i = 0; i < _numberColumnsWpc; i++)
    {
      QueryColumnInfo ci = (QueryColumnInfo) _selectedColumns.get(i);
      if (ci.getType() == QueryColumnInfo.TYPE_ID)
      {
        resultEntry[i] = columnsWpc.getOID(i + 1);
      }
      else
      {
        resultEntry[i] = columnsWpc.getObject(i + 1);
      }
    }
    //
    // Assign the CM columns to resultEntry[] - start the index at _numberColumnsWpc
    // and keep in mind - everything in the List should be Strings.
    // Just to be sure, call a String.valueOf() but care about null values.
    //
    for (int i = _numberColumnsWpc; i < _numberColumns; i++)
    {
      Object columnValue = columnsCm.get(i - _numberColumnsWpc);
      if (columnValue == null)
      {
        resultEntry[i] = null;
      }
      else
      {
        resultEntry[i] = String.valueOf(columnValue);
      }
    }
    _result.add(resultEntry);
  }

  /**
   * Moves the cursor down one entry from its current position.
   * <P>
   * A QueryResultSet cursor is initially positioned before
   * the first entry; the first call to next makes the first entry
   * the current entry.
   * <P>The second call makes the second entry the current entry, and so on.
   *
   * @return true if the new current entry is valid; false if there
   *         are no more entries
   */
  public final boolean next()
  {
    // The _currentIndex starts at index '0'
    // This function reads the entry at this position into
    // the _current variable and moves the cursor forward
    // If the cursor is already at the end, return false			
    if (_currentIndex >= _result.size())
      return false;
    _current = (Object[]) _result.get(_currentIndex++);
    return true;
  }

  /**
   * Moves the cursor to the first entry in the result set.
   *
   * @return true if the cursor is on a valid entry; false if
   *         there are no entries in the result set
   */
  public final boolean first()
  {
    // The _currentIndex and the result array starts at index '0'.
    // This function reads the first entry into the _current
    // object and moves the cursor to the next element
    // If the result set contain no entries, no changes are made.
    if (_result.size() < 1)
      return false;
    _current = (Object[]) _result.get(0);
    _currentIndex = 1;
    return true;
  }

  /**
   * Moves the cursor to the last entry in the result set.
   *
   * @return true if the cursor is on a valid row;
   * false if there are no rows in the result set
   */
  public final boolean last()
  {
    // The _currentIndex and result array starts at index '0'
    // This function reads the last entry (at size()-1) into
    // the _current object and moves the cursor one forward
    // past the end so that a call to next() would
    // result in false.
    if (_result.size() < 1)
      return false;
    _current = (Object[]) _result.get(_result.size() - 1);
    _currentIndex = _result.size();
    return true;
  }

  /**
   * Moves the cursor up one entry from its current position.
   * <P>
   *
   * @return true if the new current entry is valid; false if there
   *         are no more entries before the current one.
   */
  public final boolean previous()
  {
    // The _currentIndex starts at index '0'
    // This function reads the entry at this position into
    // the _current variable and moves the cursor forward
    // If the cursor is already at the end, return false
    if (_currentIndex <= 1)
      return false;
    _current = (Object[]) _result.get(_currentIndex--);
    return true;
  }

  /**
   * Number of entries in the result.<p>
   * This value can be used to pre-allocate an array
   * holding all results that are navigated using
   * the <TT>next()</TT> function.
   *
   * @return The number of entries in the result. Value is >= 0.
   */
  public final int size()
  {
    return _result.size();
  }

  /**
    * The number of the columns in the result.
    * The <tt>columnIndex</tt> parameter in the following calls
    * must be in the range <tt>1 .. numberColumns()</tt>.
    *
    * @return Number of columns in the result.
    */
  public final int numberColumns()
  {
    return _selectedColumns.size();
  }

  /**
   * Name of the column (header).
   * Use this name to display the column title.
   * The value of the column header is independent of the current result entry
   * position.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the header name of the column
   * @see #getTableDisplayName    
   */
  public final String getColumnDisplayName(int columnIndex)
  {
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    if (_columnDisplayNames == null)
    {
      // no results were added to the result list, therefore
      // no label names from the meta info was added
      // return the column name
      return ((QueryColumnInfo) _selectedColumns.get(columnIndex - 1)).getColumnName();

    }
    //
    // The reason why the column display name is usually retrieved from the result
    // set rather than from the QueryColumnInfo are column alias names that 
    // cannot be retrieved from the object but only from the result set meta data
    // This, however, works only if there is at least one row in the result set
    //
    return _columnDisplayNames[columnIndex - 1];
  }

  /**
   * Name of the table a column belongs to.
   * Use this name to display the column title.
   * The value of the column header is independent of the current result entry
   * position.
   * 
   * @param columnIndex
   *               the first column is 1, the second is 2, ...
   * 
   * @return the table name of the column
   * @see #getColumnDisplayName
   */
  public final String getTableDisplayName(int columnIndex)
  {
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    // no results were added to the result list, therefore
    // no label names from the meta info was added
    // return table name
    QueryColumnInfo columnInfo = (QueryColumnInfo) _selectedColumns.get(columnIndex - 1);
    return columnInfo.getTableName();
  }

  /**
   * Type of a column.
   * Although the getString() method does proper convertion, a user
   * may want to know the type to do his own formatting.
   * 
   * @param columnIndex
   *               the first column is 1, the second is 2, ...
   * 
   * @return A short value describing the column type. For the values, see the TYPE_xxx constants in class QueryColumnInfo
   * @see QueryColumnInfo
   */
  public final short getColumnType(int columnIndex)
  {
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    QueryColumnInfo columnInfo = (QueryColumnInfo) _selectedColumns.get(columnIndex - 1);
    return columnInfo.getType();
  }

  /**
   * Gets the value of a column in the current entry as a Java Boolean.
   * <br>If the value of the column is not a type compatible to a Boolean,
   * a <tt>ClassCastException</tt> is thrown.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final Boolean getBoolean(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    Object object = _current[columnIndex - 1];
    if (object == null)
      return null;
    if (object instanceof Boolean)
    {
      return (Boolean) object;
    }
    else if (object instanceof Short)
    {
      boolean bValue = ((Short) object).shortValue() != 0;
      return new Boolean(bValue);
    }
    else if (object instanceof Integer)
    {
      boolean bValue = ((Integer) object).intValue() != 0;
      return new Boolean(bValue);
    }
    else if (object instanceof Long)
    {
      boolean bValue = ((Long) object).longValue() != 0;
      return new Boolean(bValue);
    }
    else if (object instanceof BigDecimal)
    {
      boolean bValue = ((BigDecimal) object).intValue() != 0;
      return new Boolean(bValue);
    }
    throw new ClassCastException(object.getClass().getName() + " " + String.valueOf(object));
  }

  /**
   * Gets the value of a column in the current entry as an Object ID (OID).
   * <br>If the value of the column is not a type compatible to an OID,
   * a <tt>ClassCastException</tt> is thrown.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final OID getOID(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    Object object = _current[columnIndex - 1];
    if (object == null)
      return null;
    return (OID) object;
  }

  /**
   * Gets the value of a column in the current entry as a byte array.
   * <br>If the value of the column is not a type compatible to a byte array,
   * a <tt>ClassCastException</tt> is thrown.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final byte[] getBinary(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    // if the object at the column index is no byte array, a ClassCastException
    // will be thrown
    return (byte[]) _current[columnIndex - 1];
  }

  /**
   * Retrieves the value of the current tuple and the specified column as a timestamp.
   * The Calendar value contains the same timezon value that is passed in the query
   * call. If no timezone was given there, the timezone is assumed to be "UTC"
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final Calendar getTimestamp(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    // if the object at the column index is no Date, a ClassCastException
    // will be thrown
    UTCDate utcDate = (UTCDate) _current[columnIndex - 1];
    if (utcDate == null)
      return null;

    //
    // Create a calendar instance with the timezone that was used for
    // the query. If timezone is null, assume UTC
    //
    Calendar calendar =
      _timezone == null ? Calendar.getInstance(_utcTimeZone) : Calendar.getInstance(_timezone);
    //
    // set the date in UTC in the calendar object
    // the Calendar.setTime() method will itself
    // convert the UTC time to the timezone specified in
    // the getInstance() call. See the Javadoc for Calendar.setTime()
    // for details about this.
    calendar.setTime(utcDate.getDate());
    return calendar;
  }

  /**
   * Gets the value of a column in the current entry as a String value.
   * <br>If the column type is a defined constant (see <tt>QueryColumnInfo</tt>) or
   * the column type is a Timestamp, the corresponding string representation is returned.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final String getString(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    Object object = _current[columnIndex - 1];
    // Get the column info for the selected column
    QueryColumnInfo columnInfo = (QueryColumnInfo) _selectedColumns.get(columnIndex - 1);

    // If it is a null object, return null
    if (object == null)
      return null;

    if (object instanceof Integer)
    {
      // If there is an enumeration constant, return its name
      String strConstant = columnInfo.getConstantName(((Integer) object).intValue());
      if (strConstant != null)
        return strConstant;
    }
    else if (object instanceof BigDecimal)
    {
      // Oracle returns a BigDecimal where DB2 shows up with an Integer ...
      String strConstant = columnInfo.getConstantName(((BigDecimal) object).intValue());
      if (strConstant != null)
        return strConstant;
    }
    else if (object instanceof UTCDate)
    {
      UTCDate utcDate = (UTCDate) object;
      if (_timezone == null)
      {
        return JavaUtilities.convertDateToString(utcDate.getDate());
      }
      else
      {
        return JavaUtilities.convertDateToString(utcDate.getDate(), _timezone);
      }
    }
    //
    // Requesting a byte array as String is useful in the case
    // that a binary field is used to store strings, as it is
    // the case for ADOCs in the WorkItem.objectId column.
    // The Query parser accepts a syntactical enhancement
    // of the form BIN('  ') to convert a string to a binary
    // value. This function here is the counterpart: a byte array
    // is converted back to a String. Both pieces of code should
    // use the same algorithm, though.
    //
    //
    else if (object instanceof byte[])
    {
      byte[] byteArray = (byte[]) object;
      // This constructor uses the UTF-8 encoding.
      try
      {
        return new String(byteArray, "UTF-8");
      }
      catch (java.io.UnsupportedEncodingException exc)
      {
        throw new ClassCastException(exc.toString());
      }
    }
    // Else, return a string representation of the object
    return String.valueOf(object);
  }

  /**
   * Gets the value of a column in the current entry as a Long value.
   * <br>If the value of the column is not a type compatible to a Long,
   * a <tt>ClassCastException</tt> is thrown.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final Long getLong(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    Object object = _current[columnIndex - 1];
    if (object == null)
      return null;
    if (object instanceof Long)
      return (Long) object;
    if (object instanceof Short)
      return new Long(((Short) object).longValue());
    if (object instanceof Integer)
      return new Long(((Integer) object).longValue());
    // Oracle returns a BigDecimal as native object for number columns
    if (object instanceof BigDecimal)
      return new Long(((BigDecimal) object).longValue());
    throw new ClassCastException(object.getClass().getName() + " " + String.valueOf(object));
  }

  /**
   * Gets the value of a column in the current entry as an Integer value.
   * <br>If the value of the column is not a type compatible to an Integer,
   * a <tt>ClassCastException</tt> is thrown.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final Integer getInteger(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    Object object = _current[columnIndex - 1];
    if (object == null)
      return null;
    if (object instanceof Short)
      return new Integer(((Short) object).intValue());
    if (object instanceof Long)
      return new Integer(((Long) object).intValue());
    if (object instanceof Integer)
      return (Integer) object;
    // Oracle returns a BigDecimal as native object for number columns
    if (object instanceof BigDecimal)
      return new Integer(((BigDecimal) object).intValue());
    throw new ClassCastException(object.getClass().getName() + " " + String.valueOf(object));
  }

  /**
   * Gets the value of a column in the current entry as a Short value.
   * <br>If the value of the column is not a type compatible to a Short,
   * a <tt>ClassCastException</tt> is thrown.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final Short getShort(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    Object object = _current[columnIndex - 1];
    if (object == null)
      return null;
    if (object instanceof Short)
      return (Short) object;
    if (object instanceof Long)
      return new Short(((Long) object).shortValue());
    if (object instanceof Integer)
      return new Short(((Integer) object).shortValue());
    // Oracle returns a BigDecimal as native object for number columns
    if (object instanceof BigDecimal)
      return new Short(((BigDecimal) object).shortValue());
    throw new ClassCastException(object.getClass().getName() + " " + String.valueOf(object));
  }

  /**
   * Gets the value of a column in the current entry as an Object.
   * <br>If the passed <tt>columnIndex</tt> is invalid, an <tt>IndexOutOfBoundsException</tt>
   * is thrown.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public final Object getObject(int columnIndex)
  {
    // Keep in mind that columnIndex starts at '1', while
    // our array starts at '0'
    if (columnIndex < 1 || columnIndex > numberColumns())
    {
      throw new IndexOutOfBoundsException(Integer.toString(columnIndex));
    }
    return _current[columnIndex - 1];
  }

  // A list of QueryColumnInfo objects describing
  // the selected columns
  private List _selectedColumns;
  // An array of the display names for the columns
  private String[] _columnDisplayNames = null;

  // Current result entry, index starting with '0'
  private Object[] _current = null;
  // Cursor for the current entry (starts with '0')
  private int _currentIndex = 0;

  // List of resultSet entries, one entry represents one row,
  // each entry is an array of Objects
  private List _result;

  //
  // The timezone that was specified in the query() call and
  // that is now used to return timestamp values
  // If this is null, UTC time was requested.
  //
  private TimeZone _timezone = null;

  private final TimeZone _utcTimeZone = TimeZone.getTimeZone("UTC");

  private int _numberColumns = 0;
  private int _numberColumnsWpc = 0;
  private int _numberColumnsCm = 0;

  static final long serialVersionUID = 1L;
}
