/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.InitialContext;

import com.ibm.mm.sdk.common.DKAttrDefICM;
import com.ibm.mm.sdk.common.DKComponentTypeDefICM;
import com.ibm.mm.sdk.common.DKConstant;
import com.ibm.mm.sdk.common.DKConstantICM;
import com.ibm.mm.sdk.common.DKDDO;
import com.ibm.mm.sdk.common.DKFolder;
import com.ibm.mm.sdk.common.DKNVPair;
import com.ibm.mm.sdk.common.DKPidICM;
import com.ibm.mm.sdk.common.DKResults;
import com.ibm.mm.sdk.common.DKSequentialCollection;
import com.ibm.mm.sdk.common.DKUsageError;
import com.ibm.mm.sdk.common.dkIterator;
import com.ibm.mm.sdk.common.dkResultSetCursor;
import com.ibm.mm.sdk.server.DKDatastoreICM;
import com.ibm.ws.extensionhelper.ExtensionHelper;
import com.ibm.ws.extensionhelper.TransactionControl;
import com.ibm.ws.extensionhelper.TxHandle;

/**
 * Functions to access the Content Manager API.
 * This includes static functions to connect / disconnect to CM, read the
 * settings from bpecm.properties and get document content for a given itemPid.
 */
public class ContentManagerAccess
{
  /**
   * Create a Content Manager access object.
   * 
   * To use WAS connection pooling, create a file named cmbpool.ini
   * in the Content Manager cmgmt installation directory, containing
   * the following two (2) records:
   * 
   * JavaPool=DKPoolWAS
   * JDBCPrefix=jdbc/
   *
   */
  public ContentManagerAccess()
  {

    // Initialize TX handling
    try
    {
      InitialContext initialContext = new InitialContext();
      extHelper = (ExtensionHelper) initialContext.lookup("services:websphere/ExtensionHelper");
      tranControl = extHelper.getTransactionControl();
    }
    catch (Exception e)
    {
      // no error handling, as the WebSphere's emergency behaviour in this case is robust enough
    }
  }

  /**
   * Connect to Content Manager using the userid and password specified in the properties file.
   * Typically, this will connect using the icmadmin userid (or any other user with
   * admin authority) but not the userid icmconct.
   * @return A content manager connection
   * @throws IntegrationException If the given parameters did not allow to connect.
   */
  public static DKDatastoreICM connect() throws IntegrationException
  {
    DKDatastoreICM dkDatastore = null;

    try
    {
      dkDatastore = (DKDatastoreICM) AccessController.doPrivileged(new PrivilegedExceptionAction()
      {
        public Object run() throws IntegrationException
        {
          DKDatastoreICM dkDatastore = null;
          //
          // Lookup the userid and password from the properties file (password may be encrypted) 
          //
          String uid = Environment.getProperty(Environment.BPE_CM_CONNECT_UID);
          if (uid == null)
          {
            throw new IntegrationException(
              "Cannot find userid for connecting to CM DB (property name is "
                + Environment.BPE_CM_CONNECT_UID
                + ")");
          }
          String pwd =
            Environment.getPasswd(Environment.BPE_CM_CONNECT_PWD, Environment.BPE_CM_PWD_ENCRYPT);
          if (pwd == null)
          {
            throw new IntegrationException(
              "Cannot find password for connecting to CM DB (property name is "
                + Environment.BPE_CM_CONNECT_PWD
                + ")");
          }

          try
          {
            dkDatastore = connect(uid, pwd);
          }
          catch (Exception e)
          {
            // If alias has been null
            throw new IntegrationException("Cannot connect to CM DB");
          }
          return dkDatastore;
        }
      });
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    return dkDatastore;
  }

  /**
   * Connect to Content Manager with the given userid.
   * This function is as well used by the WAS User Registry implementation to check userid/password
   * during logon.
   * @param userId The userid for a Content Manager connection
   * @param password The password for the userId
   * @return A content manager connection
   * @throws IntegrationException If the given parameters did not allow to connect.
   */
  public static DKDatastoreICM connect(final String userId, final String password)
    throws IntegrationException
  {
    DKDatastoreICM dkDatastore = null;

    try
    {
      dkDatastore = (DKDatastoreICM) AccessController.doPrivileged(new PrivilegedExceptionAction()
      {
        public Object run() throws IntegrationException
        {
          DKDatastoreICM dkDatastore = null;

          try
          {
            // 
            // Connect, using the given userId, password, database name and schema name
            //
            dkDatastore = new DKDatastoreICM();
            dkDatastore.connect(getDatastoreName(), userId, password, "SCHEMA=" + getSchemaName());
          }
          catch (Exception exc)
          {
            throw new IntegrationException(exc);
          }
          return dkDatastore;
        }
      });
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    return dkDatastore;
  }

  /**
   * Disconnect from a data store
   * @param dkDatastore A connection object retrieved from connect()
   * @throws IntegrationException In case the disconnect failed
   */
  public static void disconnect(final DKDatastoreICM dkDatastore) throws IntegrationException
  {
    try
    {
      AccessController.doPrivileged(new PrivilegedExceptionAction()
      {
        public Object run() throws IntegrationException
        {
          try
          {
            if (dkDatastore != null)
            {
              dkDatastore.disconnect();
              dkDatastore.destroy();
            }
          }
          catch (Exception exc)
          {
            throw new IntegrationException(exc);
          }
          return null;
        }
      });
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
  }

  /**
   * Get a list of the item type and attribute names in Content Manager.
   * These names can be used in a UI to select corresponding attributes
   * @return A map where the item type name is the key and a list
   * of the attribute names is in the value's field.
   */
  public Map getItemTypeAttributeNames()
  {
    TxHandle globalTransaction = null;
    DKDatastoreICM dkDatastore = null;
    HashMap itemTypeAttributeNames = new HashMap();

    try
    {
      // Suspend the current global transaction to permit use of CM API

      globalTransaction = suspendGlobalTransaction();
      // Get connection for CM API from pool managed by cmbpool and WebSphere
      dkDatastore = connect();
      // Get the entities in the data store
      DKSequentialCollection pCol = (DKSequentialCollection) dkDatastore.listEntities();
      dkIterator pIter = pCol.createIterator();
      while (pIter.more() == true)
      {
        DKComponentTypeDefICM itemTypeDef = (DKComponentTypeDefICM) pIter.next();
        String itemTypeName = itemTypeDef.getName();
        // Get the attribute to the item type
        DKSequentialCollection pCol2 =
          (DKSequentialCollection) dkDatastore.listEntityAttrs(itemTypeName);
        dkIterator pIter2 = pCol2.createIterator();
        List attributeNames = new ArrayList();
        while (pIter2.more() == true)
        {
          DKAttrDefICM attrDef = (DKAttrDefICM) pIter2.next();
          attributeNames.add(attrDef.getName());
        }
        // Add itemType and its attributes to the map
        itemTypeAttributeNames.put(itemTypeName, attributeNames);
      }
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ContentManagerAccess.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return itemTypeAttributeNames;
  }

  /**
  	* Get an attribute value of items in the specified folder.
  	* This is useful when a CM folder is started on a WPC
  	* process, and the process needs to get the value of
  	* attributes of items (e.g. the license plate attribute
  	* in the adjuster's report item type).
  	* @param folderPidString - PID of folder associated with WPC process
  	* @param attributeSpecification - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
  	* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
  	* Note that &lt;attribute-name&gt; can include child component names,
  	* e.g. Book/Chapter/Section/Author.
  	* @return an Object for the attribute value or null if the attribute cannot be resolved.		
  	*/
  public Object getFolderContentAttribute(String folderPidString, String attributeSpecification)
  {
    ArrayList rows =
      getFolderContentAttributes(folderPidString, new String[] { attributeSpecification });
    Object returnValue = null;
    if (rows != null && rows.size() >= 1)
    {
      ArrayList columns = (ArrayList) rows.get(0);
      returnValue = columns.get(ATTRIBUTE_VALUE_COLUMN);
    }
    return returnValue;
  }

  /**
  	* Get an attribute value of the associated item (when it is no folder).
  	* @param itemPidString - PID of item containing attributes.
  	* @param attributeSpecification - Attribute to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
  	* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
  	* Note that &lt;attribute-name&gt; can include child component names,
  	* e.g. Book/Chapter/Section/Author.
  	* @return an Object for the attribute value or null if the attribute cannot be resolved.
  	*/
  public Object getContentAttribute(String itemPidString, String attributeSpecification)
  {
    ArrayList rows = getContentAttributes(itemPidString, new String[] { attributeSpecification });
    Object returnValue = null;
    if (rows != null && rows.size() >= 1)
    {
      ArrayList columns = (ArrayList) rows.get(0);
      returnValue = columns.get(ATTRIBUTE_VALUE_COLUMN);
    }
    return returnValue;
  }

  /**
   * Get the attributes of items in the specified folder.
   * This is useful when a CM folder is started on a WPC
   * process, and the process needs to get the value of
   * attributes of items (e.g. the license plate attribute
   * in the adjuster's report item type).
   * @param folderPidString - PID of folder associated with WPC process
   * @param attributeSpecifications - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
   * format, e.g.
   * <code><pre>
   * new String[]
   * {
   *     "XYZ_ClaimForm/XYZ_DriversLic",
   *     "XYZ_ClaimForm/XYZ_PolicyNum",
   *     "XYZ_AdjReport/XYZ_LicPlate"
   * }
   * </pre></code>
   * Note that &lt;attribute-name&gt; can include child component names,
   * e.g. Book/Chapter/Section/Author.
   * @return an ArrayList of ArrayLists (e.g. a table) containing
   * one row for each returned attribute and columns for item type
   * name, item pid string, attribute name and attribute value.
   */
  public ArrayList getFolderContentAttributes(
    String folderPidString,
    String[] attributeSpecifications)
  {
    ArrayList table = null;
    try
    {
      String itemId = new DKPidICM(folderPidString).getItemId();
      String query =
        "/*[@ITEMID=\"" + itemId + "\"]/OUTBOUNDLINK[@LINKTYPE=\"DKFolder\"]/@TARGETITEMREF => *[@VERSIONID = latest-version(.)]";
      table = retrieveAttributes(query, attributeSpecifications);
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    return table;
  }

  /**
   * Get the attributes of of the specified item.
   * @param itemPidString - PID of item containing attributes.
   * @param attributeSpecifications - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; format, e.g.
   * <code><pre>
   * new String[]
   * {
   *     "XYZ_ClaimForm/XYZ_DriversLic",
   *     "XYZ_ClaimForm/XYZ_PolicyNum",
   *     "XYZ_AdjReport/XYZ_LicPlate"
   * }
   * </pre></code>
   * Note that &lt;attribute-name&gt; can include child component names,
   * e.g. Book/Chapter/Section/Author.
   * @return an ArrayList of ArrayLists (e.g. a table) containing
   * one row for each returned attribute and columns for item type
   * name, item pid string, attribute name and attribute value.
   */
  public ArrayList getContentAttributes(String itemPidString, String[] attributeSpecifications)
  {
    ArrayList table = null;
    try
    {
      String itemId = new DKPidICM(itemPidString).getItemId();
      String query = "/*[@ITEMID=\"" + itemId + "\"]";
      table = retrieveAttributes(query, attributeSpecifications);
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    return table;
  }

  public ArrayList retrieveAttributes(String query, String[] attributeSpecifications)
  {
    ArrayList rows = null;
    TxHandle globalTransaction = null;
    DKDatastoreICM dkDatastore = null;

    try
    {
      // Suspend the current global transaction to permit use of CM API
      globalTransaction = suspendGlobalTransaction();

      // Get connection for CM API from pool managed by cmbpool and WebSphere
      dkDatastore = connect();

      // Get the (potentially cached) query options for the attribute specs
      QueryOptions queryOptions = QueryOptions.getQueryOptions(attributeSpecifications);

      DKNVPair options[] = queryOptions.getOptions();
      Map itemTypes = queryOptions.getItemTypes();

      // Execute the query
      dkResultSetCursor results =
        dkDatastore.execute(query, DKConstant.DK_CM_XQPE_QL_TYPE, options);

      DKDDO ddo = null;
      rows = new ArrayList();

      // For each item instance returned by the query
      while ((ddo = results.fetchNext()) != null)
      {
        // Get the persistent identifier for this item instance
        DKPidICM itemPid = (DKPidICM) ddo.getPidObject();

        // Get the item type name for this item instance
        String itemTypeName = itemPid.getObjectType();

        // Get the string representation of this persistent identifier
        String itemPidString = itemPid.toString();

        // The query results may contain system data (e.g. ICMParts<nnnn>)
        DKSequentialCollection attributeNames =
          (DKSequentialCollection) itemTypes.get(itemTypeName);

        // The projection list appears to be a filtering specification,
        // not a retrieval specification. All attributes are returned for item
        // types that match the query, but don't match the projection list.

        //Assert.assertion(attributeNames != null, "Unrequested item type");
        if (attributeNames != null)
        {
          // So we extract what we want rather than saving everything
          dkIterator attributeNameIterator = attributeNames.createIterator();

          // While there are requested attributes for this item type        
          while (attributeNameIterator.more())
          {
            // Get the attribute name
            String attributeName = (String) attributeNameIterator.next();

            // Get the attribute's value
            Object attributeValue = ddo.getDataByName(attributeName);

            // If the attribute was available in the query results
            if (attributeValue != null)
            {
              // Save the attribute details as a row (of columns) in the table
              ArrayList columns = new ArrayList();
              rows.add(columns);
              columns.add(ITEM_TYPE_COLUMN, itemTypeName);
              columns.add(PID_STRING_COLUMN, itemPidString);
              columns.add(ATTRIBUTE_NAME_COLUMN, attributeName);
              columns.add(ATTRIBUTE_VALUE_COLUMN, attributeValue);
            }
          }
        }
      }
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ContentManagerAccess.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return rows;
  }

  /**
   * Evaluate a Content Manager XPath expression.
   * Use this function in a Process Choreographer transition condition to
   * determine a flow route based on document contents.
   * @param xPath A Content Manager XPath expression for a condition
   * @return true, if the condition is true
   */
  public boolean evaluateCondition(String xPath)
  {
    boolean isConditionTrue = false;
    DKDatastoreICM dkDatastore = null;
    TxHandle globalTransaction = null;
    try
    {
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ContentManagerAccess.connect();
      DKNVPair options[] =
        new DKNVPair[] {
          new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "1"),
          new DKNVPair(
            DKConstant.DK_CM_PARM_RETRIEVE,
            new Integer(DKConstant.DK_CM_CONTENT_IDONLY)),
          new DKNVPair(DKConstant.DK_CM_PARM_END, null)};
      DKResults results =
        (DKResults) dkDatastore.evaluate(xPath, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      isConditionTrue = results.cardinality() > 0;
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ContentManagerAccess.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return isConditionTrue;
  }

  /**
   * Suspend an existing global transaction.
   * @return The transaction object. Use this object to resume the transaction via <code>resumeGlobalTransaction</code>.
   */
  protected TxHandle suspendGlobalTransaction()
  {
    TxHandle transaction = null;
    try
    {
      transaction = tranControl.preinvoke(true, false);
    }
    catch (javax.transaction.SystemException exc)
    {
      // Here, we can either continue processing or rethrow the exception.
      // Assuming that following JDBC calls may be successful even if the above exception was
      // thrown, we continue processing.
    }
    catch (javax.transaction.NotSupportedException exc)
    {
      // Here, we can either continue processing or rethrow the exception.
      // Assuming that following JDBC calls may be successful even if the above exception was
      // thrown, we continue processing.
    }
    return transaction;
  }

  /**
   * Resume a previously suspended global transaction.
   * @param transaction The transaction object returned by <code>suspendGlobalTransaction()</code>.
   */
  protected void resumeGlobalTransaction(final TxHandle transaction)
  {
    try
    {
      tranControl.postinvoke(transaction); // resumes previous global or local UOW
    }
    catch (javax.transaction.SystemException exc)
    {
      // Here, we can either continue processing or rethrow the exception.
      // Assuming that following JDBC calls may be successful even if the above exception was
      // thrown, we continue processing.
    }
    catch (javax.transaction.InvalidTransactionException exc)
    {
      // This exception is considered more severe than the SystemException during suspend above,
      // since we retrieved a non-null transaction object that now turns out to be invalid.
      throw IntegrationException.wrap(exc);
    }
    catch (com.ibm.ws.extensionhelper.exception.InconsistentTransactionException exc)
    {
      // This exception is considered more severe than the SystemException during suspend above,
      // since we retrieved a non-null transaction object that now turns out to be invalid.
      throw IntegrationException.wrap(exc);
    }
  }

  /**    
   * @return The Content manager data store name (e.g. ICMNLSDB)
   */
  public static String getDatastoreName()
  {
    if (_datastoreName == null)
    {
      synchronized (DATASTORE_NAME_PROPERTY)
      {
        _datastoreName = Environment.getProperty(DATASTORE_NAME_PROPERTY);
        if (_datastoreName == null)
        {
          throw new IntegrationException(
            "Set "
              + DATASTORE_NAME_PROPERTY
              + " in bpecm.properties to the datastore name (e.g. ICMNLSDB).");
        }
      }
    }
    return _datastoreName;
  }

  /**   
   * @return The schema name for the Content Manager connect string SCHEMA=xxxx
   */
  public static String getSchemaName()
  {
    if (_schemaName == null)
    {
      synchronized (SCHEMA_NAME_PROPERTY)
      {
        _schemaName = Environment.getProperty(SCHEMA_NAME_PROPERTY);
        if (_schemaName == null)
        {
          throw new IntegrationException(
            "Set "
              + SCHEMA_NAME_PROPERTY
              + " in bpecm.properties to the schema name for a Content Manager connection (e.g. ICMADMIN)");
        }
      }
    }
    return _schemaName;
  }

  /**
   * Add the results of a Content Manager query to the folder that is routed through the process.
   * 
   * @param folderPidString PID string of the folder that is routed through the process
   * @param query XPath query string that specifies the items to be added
   * @return number of items that have been added to the folder (number of hits)
   */

  public int addItemToFlowFolder(String folderPidString, String query)
  {
    DKDatastoreICM dkDatastore = null;
    TxHandle globalTransaction = null;
    int numberOfHits = 0;
    try
    {
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = connect();
      DKNVPair options[] = new DKNVPair[2];
      options[0] =
        new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE, new Integer(DKConstant.DK_CM_CONTENT_IDONLY));
      options[1] = new DKNVPair(DKConstant.DK_CM_PARM_END, null);
      dkResultSetCursor cursor =
        dkDatastore.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      numberOfHits = cursor.cardinality();
      if (numberOfHits > 0)
      {
        DKDDO ddoFolder = dkDatastore.createDDO(folderPidString);
        ddoFolder.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND);
        // Outbound Links must be specified.

        short dataid = ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR, DKConstant.DK_CM_DKFOLDER);

        if (dataid == 0)
          throw new IntegrationException("Flow item must be a folder for 'addItemToFlowFolder' to apply.");

        DKFolder dkFolder = (DKFolder) ddoFolder.getData(dataid);
        DKDDO ddo; // no need to check out the folder as this only updates the links table
        try
        {
          while ((ddo = cursor.fetchNext()) != null)
            dkFolder.addElement(ddo);
          ddoFolder.update();
        }
        catch (DKUsageError ue)
        {
          System.err.println("addItemToFLowFolder: " + ue.getMessage());
        }
      }
      cursor.destroy();
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ContentManagerAccess.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return numberOfHits;
  }

  /**
   * Add the item routed through the process to the folder retrieved by a Content Manager query. 
   * 
   * @param pidString PID string of the item that is routed through the process
   * @param query XPath query string that specifies the folder to which this item should be added
   * @return number of folders that matched the query
   */

  public int addFlowItemToFolder(String pidString, String query)
  {
    DKDatastoreICM dkDatastore = null;
    TxHandle globalTransaction = null;
    int numberOfHits = 0;
    try
    {
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = connect();
      DKNVPair options[] = new DKNVPair[2];
      options[0] =
        new DKNVPair(
          DKConstant.DK_CM_PARM_RETRIEVE,
          new Integer(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND));
      options[1] = new DKNVPair(DKConstant.DK_CM_PARM_END, null);
      dkResultSetCursor cursor =
        dkDatastore.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      numberOfHits = cursor.cardinality();
      if (numberOfHits > 0)
      {
        DKDDO ddoFolder = cursor.fetchNext();
        short dataid = ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR, DKConstant.DK_CM_DKFOLDER);

        if (dataid == 0)
          throw new IntegrationException("Retrieved item must be a folder for 'addFlowItemToFolder' to apply.");

        DKFolder dkFolder = (DKFolder) ddoFolder.getData(dataid);
        DKDDO ddo = dkDatastore.createDDO(pidString);
        // no need to check out the folder as this only updates the links table
        try
        {
          dkFolder.addElement(ddo);
          dkDatastore.createDDO(pidString);
          ddoFolder.update();
        }
        catch (DKUsageError ue)
        {
          System.err.println("addFlowItemToFolder: " + ue.getMessage());
        }
      }
      cursor.destroy();
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ContentManagerAccess.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return numberOfHits;
  }

  // Name of the data store and its corresponding property in bpecm.properties
  private static String _datastoreName = null;
  private static final String DATASTORE_NAME_PROPERTY = "ContentManagerDatastoreName";

  // Name of the schema name its corresponding property in bpecm.properties
  private static String _schemaName = null;
  private static final String SCHEMA_NAME_PROPERTY = "ContentManagerSchema";

  // Some variables for allowing local transactions
  ExtensionHelper extHelper = null;
  TransactionControl tranControl = null;

  // Index numbers for columns in returned attribute tables. Must be kept in order of initialization.
  public final static int ITEM_TYPE_COLUMN = 0;
  public final static int PID_STRING_COLUMN = 1;
  public final static int ATTRIBUTE_NAME_COLUMN = 2;
  public final static int ATTRIBUTE_VALUE_COLUMN = 3;

}
