/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.io.IOException;
import java.io.InputStream;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Properties;

public class Environment
{

  public static final String COPYRIGHT = "\n\n(C) Copyright IBM Corporation 2001, 2005.\n\n";

  /**
     * Initiates to search the properties in bpecm.properties. 
     * These properties are overwritten with the properties
     * found in the system properties.
   */
  private static Properties createEnvironment()
  {
    Properties env = new Properties();

    Properties prop = getFromClassPath(BPE_CM_PROPERTIES);

    if (prop != null)
      env.putAll(prop);

    try
    {
      // Requires the following permission:
      //   permission java.util.PropertyPermission "*", "read, write";
      prop = (Properties) AccessController.doPrivileged(new PrivilegedAction()
      {
        public Object run()
        {
          return System.getProperties();
        }
      });
      if (prop != null)
        env.putAll(prop);
    }
    catch (SecurityException e)
    {
      System.out.println("Cannot access system properties.");
    }

    return env;
  }

  /**
   *  Trys to locate the properties file in the classpath and returns its contents in a Properties Object.
   *
   *  @param propertiesFileName
   */
  private static Properties getFromClassPath(final String propertiesFileName)
  {
    Properties prop = null;

    try
    {
      InputStream in = (InputStream) AccessController.doPrivileged(new PrivilegedAction()
      {
        public Object run()
        {
          InputStream is = ClassLoader.getSystemResourceAsStream(propertiesFileName);

          if (is == null)
            is = Environment.class.getClassLoader().getResourceAsStream(propertiesFileName);

          return is;
        }
      });
      if (in != null)
      {
        prop = new Properties();
        prop.load(in);
        in.close();
      }
    }
    catch (IOException e)
    {
      System.out.println(
        "Cannot get environment from classloader/properties file: " + e.toString());
      return null;
    }
    return prop;
  }

  private static Object get(String key)
  {
    Object value = null;

    if (singleton == null)
    {
      synchronized (Environment.class)
      {
        if (singleton == null)
          singleton = createEnvironment();
      }
    }

    if (singleton != null)
      value = singleton.get(key);

    return value;
  }

  /**
  * Looks up the requested Property. For details about the lookup strategy, please see the class description.
  * @param key
  *   The properties key. Use one of the static Strings defined in this class.
  * @return
  *   The property, or null if it is not found.
  */
  public static String getProperty(String key)
  {
    return (String) get(key);
  }

  /**
  * Looks up a password entry.
  * @param pwdKey
  *   The properties key for the password. Use one of the static Strings defined in this class.
  * @param encryptionKey
  *   The properties key for the encryption method. Do not confuse with ENCRYPTION KEY!
  * 
  * @return
  *   The decrypted password, or null if it is not found.
  */
  public static String getPasswd(String pwdKey, String encryptionKey)
  {
    String pwdValue = (String) get(pwdKey);
    if (pwdValue == null)
      return null;

    String encryptionValue = (String) get(encryptionKey);
    if (encryptionValue == null || encryptionValue.equals("") || encryptionValue.equals("0"))
    {
      // no encryption
      if (encryptionValue == null)
      {
        System.out.println(
          "Property '"
            + BPE_CM_PWD_ENCRYPT
            + "' not found in "
            + BPE_CM_PROPERTIES
            + " - password assumed to be unencrypted.");
      }
      return pwdValue;
    }
    else if (encryptionValue.equals("1"))
    {
      // use the sample encryption as defined in method encryption_1() below  
      return encryption_1(pwdValue);
    }
    else
    {
      throw new IntegrationException(
        "Invalid value '"
          + encryptionValue
          + "' found in "
          + BPE_CM_PROPERTIES
          + " (property name is "
          + encryptionKey
          + ")");
    }
  }

  /**
   * @param pwdValue - the encrypted passwd
   * @return unencrypted passwd
  */

  private static String encryption_1(String pwdValue)
  {
    // Here, we show (just for demo purposes) how you could plug in your favorite encryption method.
    // Unfortunately we cannot use one-way encryption and compare the encrypted passwords, since
    //  we need the readable password for connecting to CM.  
    // The method used here is "rot13" encryption:
    //  - for each character, add an offset of 13 (if this is still a char), otherwise subtract 13. 
    //  - very easy to understand
    //  - encryption and decryption are identical
    //  - you can do the encryption (for the properties file) just with paper and pencil
    //  - usually, someone seeing a rot13 encrypted password cannot memorize it

    //  Input char:    abcdefghijklm nopqrstuvwxyz ABCDEFGHIJKLM NOPQRSTUVWXYZ    
    // Output char:    nopqrstuvwxyz abcdefghijklm NOPQRSTUVWXYZ ABCDEFGHIJKLM

    StringBuffer result = new StringBuffer(pwdValue);
    int len = result.length();
    for (int i = 0; i < len; i++)
    {
      char curChar = result.charAt(i);
      char newChar = curChar; // default; keep non-alpha chars constant 
      // disjoint branches, performance is not an issue here
      if (curChar >= 'a' && curChar <= 'm') newChar += 13;
      if (curChar >= 'n' && curChar <= 'z') newChar -= 13;
      if (curChar >= 'A' && curChar <= 'M') newChar += 13;
      if (curChar >= 'N' && curChar <= 'Z') newChar -= 13;
      if (newChar != curChar) result.setCharAt(i, newChar);
    }
    return new String(result);
  }

  /*******************************************************************************/
  /* KEYS                                                                        */
  /*******************************************************************************/

  /**
  * Key for the JNDI name of the internal input queue. Derived from WCCM's input listener port (server-pme.xml)
  * in production environments.
  */
  public static final String INTERNAL_QUEUE_JNDI_NAME = "InternalQueueJNDIName";
  /**
  * Keys for the userid and password to connect to CM database.
  */
  public final static String BPE_CM_CONNECT_UID = "ContentManagerAuthenticationUid";
  public final static String BPE_CM_CONNECT_PWD = "ContentManagerAuthenticationPwd";
  public final static String BPE_CM_PWD_ENCRYPT = "ContentManagerAuthenticationPwdEncryption";

  /*******************************************************************************/
  /*  private constants                                                          */
  /*******************************************************************************/
  private final static String BPE_CM_PROPERTIES = "bpecm.properties";

  /*****************************************************************************/
  /* private variables                                                         */
  /*****************************************************************************/
  private static Properties singleton;
}
