/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.mm.sdk.common.DKConstant;
import com.ibm.mm.sdk.common.DKNVPair;
import com.ibm.mm.sdk.common.DKSequentialCollection;
import com.ibm.mm.sdk.common.DKUsageError;

class QueryOptions
{
  QueryOptions(DKNVPair[] options, Map itemTypes)
  {
    _options = options;
    _itemTypes = itemTypes;
  }

  Map getItemTypes()
  {
    return _itemTypes;
  }

  DKNVPair[] getOptions()
  {
    return _options;
  }

  Map _itemTypes;
  DKNVPair[] _options;

  /**
   * A Hashmap for attribute specifications to query options
   */
  private static Map _queryOptions = new HashMap();

  /**
   * Get the query options for a subsequent CM query for one or more
   * attribute specifications.
   * This function is used both for combined query and retrieving attribute values. 
   * @param attributeSpecifications
   * @return The query options (contains the DKNVPair array and the itemTypes map)
   * @throws DKUsageError
   */
  public static QueryOptions getQueryOptions(String[] attributeSpecifications) throws DKUsageError
  {
    // 
    // Caching - create a string from the list of attribute specifications
    // This will be used as the key to a HashMap which stores a mapping
    // from the attribute specification to the query options which are used
    // for the query call.
    // It is assumed that a particular query is run multiple times, thus
    // it is worth to cache this computation.	  
    // 
    StringBuffer lookupKey = new StringBuffer();
    for (int i = 0; i < attributeSpecifications.length; i++)
    {
      lookupKey.append(attributeSpecifications[i]);
    }
    String lookupKeyString = lookupKey.toString();
    // check the (static) cache if the corresponding query options already have been computed
    QueryOptions queryOptions = null;
    synchronized (_queryOptions)
    {
      queryOptions = (QueryOptions) _queryOptions.get(lookupKeyString);
    }
    if (queryOptions == null)
    {
      // not yet found in cache - compute the query options for these
      // This keeps track of each of the unique item types in the attribute list

      HashMap itemTypes = new HashMap();
      // For each attribute in the attribute specification... 
      for (int i = 0; i < attributeSpecifications.length; i++)
      {
        // Get the attribute specification of the form <item-type>/<attribute-name>
        String attributeSpecification = attributeSpecifications[i];

        // Get the location of the delimiter between the item type and attribute name
        int delimiter = attributeSpecification.indexOf('/');
        if (delimiter == -1)
          throw new IntegrationException("Missing delimiter. Attribute name must be of form \"item-type/attribute-name\"");

        // Get the text up to the delimiter as the item type name
        String itemTypeName = attributeSpecification.substring(0, delimiter);

        // Get the text after the delimiter as the attribute name (which may contain more slashes)
        String attributeName = attributeSpecification.substring(delimiter + 1);

        // Get the attribute names (for this item type) that we've seen so far
        DKSequentialCollection attributeNames =
          (DKSequentialCollection) itemTypes.get(itemTypeName);

        // If this is the first attribute name for this item type
        if (attributeNames == null)
        {
          // Allocate a new collection to hold the attribute names for this item type
          attributeNames = new DKSequentialCollection();

          // Save the collection so it can be retrieved later by item type name
          itemTypes.put(itemTypeName, attributeNames);
        }

        // Add this attribute name to the collection of attributes for this item type
        attributeNames.addElement(attributeName);
      }
      // Allocate an array of ordered pairs (item type, collection of attribute names for item type)
      DKNVPair[] projectionList = new DKNVPair[itemTypes.size()];
      int projectionListIndex = 0;

      // For each unique item type in the attribute specification (note that we have now coalesced
      // multiple possible attribute specifications into one per item type)
      for (Iterator itemTypeIterator = itemTypes.keySet().iterator(); itemTypeIterator.hasNext();)
      {
        // Get the name of the item type
        String itemTypeName = (String) itemTypeIterator.next();

        // Get the collection of attribute names for this item type
        DKSequentialCollection attributeNames =
          (DKSequentialCollection) itemTypes.get(itemTypeName);

        // Create an ordered pair to hold the item type and attribute name collection
        projectionList[projectionListIndex++] = new DKNVPair(itemTypeName, attributeNames);
      }

      // Initialize the options for the query
      DKNVPair[] options =
        new DKNVPair[] {
          new DKNVPair(DKConstant.DK_CM_PARM_PROJECTION_LIST, projectionList),
          new DKNVPair(DKConstant.DK_CM_PARM_END, null)};

      // Create a new query options object to store both the options and the item type
      queryOptions = new QueryOptions(options, itemTypes);
      // Add the resulting query options to the cache for later rerun of the very query
      synchronized (_queryOptions)
      {
        _queryOptions.put(lookupKeyString, queryOptions);
      }
    }
    return queryOptions;
  }
}