/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.utils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.ibm.bpe.api.BusinessProcess;
import com.ibm.bpe.api.BusinessProcessHome;
import com.ibm.mm.beans.CMBConnection;
import com.ibm.mm.beans.CMBDataManagement;
import com.ibm.mm.beans.CMBItem;

/**
 * This class provides some global methods available to the JSPs and Servlets of the Quick Start Client for simplification
 * and generalization purposes.
 */

public class Helpers {

 /**
  * This method looks up the business process bean from JNDI and connects with the Content Manager datastore
  * using an LTPA token. It obtains the userId from the caller subject, stores it in the session attribute
  * "UserId", stores the LTPA token in the session attribute "credToken" and the CM connection in the session
  * attribute "connection".
  * @param request - current HttpRequest 
  * @return connection if initialization was successful; null otherwise
  */
 
 static public CMBConnection initialize(HttpServletRequest request) {
 
	CMBConnection connection = null;

	try {
		HttpSession session = request.getSession();

		//	Lookup the home interface of the BusinessProcess bean and make it available to the Servlets/JSPs that use it
		javax.naming.InitialContext context = new javax.naming.InitialContext();
		Object obj = context.lookup("java:comp/env/ejb/BusinessProcess");
		BusinessProcessHome processHome = (BusinessProcessHome)javax.rmi.PortableRemoteObject.narrow(obj, BusinessProcessHome.class);
		BusinessProcess process = processHome.create();
		session.setAttribute("ProcessService", process);

		javax.security.auth.Subject currentSubject = com.ibm.websphere.security.auth.WSSubject.getCallerSubject();
		if (currentSubject == null)
			System.err.println("Unable to get Caller Subject. Authentication Failed.");
		else {
			// Extract the userId from the WSCredential object and store it in a session attribute
			com.ibm.websphere.security.cred.WSCredential credential = 
					(com.ibm.websphere.security.cred.WSCredential)currentSubject.getPublicCredentials(com.ibm.websphere.security.cred.WSCredential.class).iterator().next();
			String userId = credential.getSecurityName();
			session.setAttribute("UserId", userId);

			// look up the Ltpa token and store it in a session attribute
			javax.servlet.http.Cookie cookies[] = request.getCookies();
			String credentialToken = null;
			for (int i = 0; i < cookies.length; i++) {
				if (cookies[i].getName().equalsIgnoreCase("LtpaToken")) {
					credentialToken = cookies[i].getValue();
					break;
				}
			}
			if (credentialToken == null) throw new Exception("Unable to obtain credential token. Make sure cookies are enabled.");

			// set up CM connection and connect using the LTPA token
			connection = new CMBConnection();
			connection.setDsType(ProcessData.CM_DATASTORE_TYPE);
			connection.connectWithCredential(ProcessData.CM_DATASTORE_NAME, credentialToken, "CHKDBPOOL=NO");
			session.setAttribute("credToken", credentialToken);
			session.setAttribute("connection", connection);
		}
	}
	catch (Exception e) {
		e.printStackTrace();
	}
	
	return connection;
 }

 /**
  * Re-establish a timed-out library server connection or re-initialize session if connection
  * is not available as session attribute.
  * @param request the current HTTP request
  * @throws Exception connection or session re-initialization has failed
  * @return (re-)validated library server connection
  */
 
 static public CMBConnection validateConnection(HttpServletRequest request) throws Exception {
 	
	HttpSession session = request.getSession();
	CMBConnection connection = (CMBConnection) session.getAttribute("connection");
	if (connection == null) {
		connection = initialize(request);
	}
	else if (!connection.isConnected()) {
		String credentialToken = (String)session.getAttribute("credToken");
		connection.connectWithCredential(ProcessData.CM_DATASTORE_NAME, credentialToken, "CHKDBPOOL=NO");
		System.out.println("-> reconnected");
	}
	return connection;
 }
 
 /**
  * This method creates, initializes and returns an object of class CMBDataManagement 
  * which is associated with the CMBItem object specified by itemId. 
  * @param request current HttpRequest 
  * @param itemId itemId of the item to be located
  * @param retrieve - true, if the most recent version of the item should be retrieved from the datastore, false otherwise   
  * @return Returns an object of class CMBDataManagement which is associated to the item specified by itemId
  * @throws Exception library server can not be accessed or item can not be located
  */
 
 static public CMBDataManagement setupDataBean(HttpServletRequest request,
 											   String itemId,
 											   boolean retrieve) throws Exception {
	HttpSession session = request.getSession();

	// check and, if necessary, re-establish the library server connection
	// header.jsp does this for the JSPs - the servlets either perform an initial setupDataBean
	// or call validateConnection() explicitly
	CMBConnection connection = validateConnection(request);
	
	// initialize the data management bean
	CMBDataManagement 	dataMgmt = connection.getDataManagement();
	// create a new item for the given id and assign it to the data management bean
	CMBItem itemForPart = connection.getObjectManagement().createItem(itemId);
	dataMgmt.setDataObject(itemForPart);
	// if needed: retrieve the latest version of the item from the datastore
	if (retrieve) dataMgmt.retrieveItem();
	
	return dataMgmt;
  }
  
  /**
   * This method is specific to using the client with the Content Manager First Steps Sample Data as these
   * contain very long display names that include a description of the type "(Content Manager version 8 sample data)".
   * This method truncates the length of a display name by remvoing the comments enclosed by parenthesis.
   * If no parenthesis is present and the name consists of more than 50 characters as in the case of XYZ_ClaimFolder),
   * the display name is truncated after the 24th character.
   * This method may be disabled when using the client with custom data.
   * 
   * @param name item type or attribute name to be truncated
   * @return result of truncation
   */
  
  static public String prepareName(String name) {
  	String preparedName = "";
  	if (name != null) {
  		int endPos = name.indexOf("(") - 1;
  		if (endPos < 0)
  			endPos = name.length();
  		else if (endPos > ProcessData.DISPLAY_NAME_SIZE_LIMIT)
  			endPos = ProcessData.DISPLAY_NAME_TRUNCATION_POS;  // this applies to XYZ_ClaimFolder
  		preparedName = name.substring(0, endPos);
  	}
  	return preparedName;
  }
  
  /**
   * Create a HTML definition of the input field of an HTML form. Depending on the size of the input field it is either a
   * field of type INPUT or of type TEXTAREA. If it is of type TEXTAREA the heigth of the field is adjusted so the width
   * does not exceed 80 characters.
   * @param cols size of the input field
   * @param fieldName name of the input field
   * @param fieldValue value to be displayed in the input field
   * @return HTML definition of a corresponding field of type INPUT or TEXTAREA. In case of TEXTAREA it includes as many lines
   * as necessary not to exceed a maximum length of 80 characters
   */
  
  final static int TEXTAREA_WIDTH = 60;  // width of a textarea 
  
  static public String defineInputField(int cols,
  									    String fieldName,
  									    String fieldValue)
  {	int rows = 0;
	if (cols > TEXTAREA_WIDTH) {
	   rows = (cols / TEXTAREA_WIDTH) + 1;
	   cols = TEXTAREA_WIDTH;
	}
	return (rows == 0) ?
			"<INPUT type=\"text\" name=\"" + fieldName + "\" value=\"" + fieldValue + "\" size=\"" + cols + "\">" :
			"<TEXTAREA name=\"" + fieldName + "\" rows=\"" + rows + "\" cols=\"" + cols + "\">" + fieldValue + "</TEXTAREA></TD>";
  }
  
} // Helpers
